(* ::Package:: *)

(*:Mathematica Version: 8.01 *)

(*:Package Version: 4.01 *)

(*:Version History: -.-- (1998.07.12 : Dirichlet.m)
                    1.01 (2004.03.15 - 2006.06.16)
                    1.02 (2006.06.17 - 2006.06.18)
                    1.03 (2006.06.19 - 2006.07.07)
                    2.01 (2006.07.08 - 2008.09.14)
                    2.02 (2008.09.15 - 2011.04.30)
                    2.03 (2011.05.01 - 2012.12.12)
                    2.04 (2012.12.13 - 2014.08.06)
                    2.05 (2014.08.07 - 2015.01.24)
                    2.06 (2015.01.25 - 2015.02.22)HurwitzZeta, DirichletL
                    2.07 (2015.02.23 - 2015.03.03)Ramanujan, Stieltjes, LandauHurwitz
                    3.01 (2015.03.04 - 2015.04.16)RiemannPrimeR
                    3.02 (2015.04.17 - 2016.02.27)LerchPhi
                    3.03 (2016.02.28 - 2016.05.02)DedekindZeta, Eichler
                    3.04 (2016.05.03 - 2016.06.17)EulerSum
                    3.05 (2016.06.18 - 2017.09.17)Riesz, Common-codes
                    3.06 (2017.09.18 - 2018.02.04)PrimeNumberZetaP
                    3.07 (2018.02.05 - 2018.04.01)Clausen, RogersL, Debye, etc.
                    3.08 (2018.04.02 - 2019.07.02)Debye, Grueneisen, etc.
                    3.09 (2019.07.03 - 2020.01.10)GlaisherRamanujan(->Automorphic.m)
                    4.01 (2020.01.11 -     .  .  )RiemannVonMangoldtPi *)

(*:Copyright: Copyright 1998.07-2020.01, Souichiro Ikebe *)

(*:Author: Souichiro Ikebe *)

(*:Name: SpecialFunctions`Zeta` *)

(*:Title: Zeta, Dirichlet, RiemannPrimeR, RieszR *)

(*:Keywords: RiemannXi, PeriodicZetaF, ImprovedHurwitzZeta, 
             ImprovedDirichletL, GaussSum, ConjugateChiIndex, 
             RealDirichletCharacterQ, PrimitiveDirichletCharacterQ, 
             GrundzahlD, GrundzahlDQ, QuadraticFieldDedekindZeta, 
             CyclotomicFieldDedekindZeta, ImprovedLerchPhi, 
             ZetaPrime, StieltjesFGamma, SitaramachandraraoDelta, 
             ZetaZeta, LehmerZetaZeta, 
             BellExponentB, BellExponentB2, 
             PrimeNumberZetaP, PrimeNumberZetaP2, 
             RamanujanLTau, RamanujanSiegelTheta, RamanujanSiegelZ, 
             EichlerEllipticCurveL, EichlerCoefficientC, FibonacciZeta, 
             RiemannPrimeR, RiemannPrimeRExp, RiemannExpIntegral, 
             RiemannLogIntegral, RiemannVonMangoldtPi, 
             RiemannVonMangoldtPiExact, ChebyshevPrimeTheory, ChebyshevPsi, 
             EulerSumH, RieszR, HardyLittlewoodR, 
             ClausenCL2, PeriodicClausenCL2, 
             GeneralClausenC, GeneralClausenS, 
             PeriodicGeneralClausenC, PeriodicGeneralClausenS, 
             RogersL, ArcTanIntegralTi, DebyeD1, DebyeD2, 
             GrueneisenG1, GrueneisenG2, StroemgrenPhi1, StroemgrenPhi2 *)

(*:Summary:
 
  This package implements the functions related zeta function, 
  Dirichlet series, Stieltjes function, polylogarithms. 
  And incidental to, also implements of the Riemann's prime-
  counting function R(x) & Riemann-Von Mangoldt's formula. *)

(*****************************************************************************)






BeginPackage["SpecialFunctions`Zeta`"]






RiemannXi::usage =
"RiemannXi[s] gives the value of the Riemann's xi function.
 Xi[s]=s*(s-1)*Pi^(-s/2)*Gamma[s/2]*Zeta[s]."

PeriodicZetaF::usage =
"PeriodicZetaF[s,a] gives the value of the periodic zeta F-function.
 F[s,a]=Sum[Exp[2 Pi I k a]/k^s,{k,1,Infinity}]."

ImprovedHurwitzZeta::usage =
"ImprovedHurwitzZeta[s,a] gives the value of the Hurwitz zeta function.
 This function, improved the same function of Mathematica at Re[s]<-3."

ImprovedDirichletL::usage =
"ImprovedDirichletL[m,j,s] gives the value of the Dirichlet L-function, 
 with modulas m and index j. 
 This function, improved the same function of Mathematica at Re[s]<-2."

GaussSum::usage =
"GaussSum[m,j] gives the value of the Gauss's sum, with modulas m and 
 index j."

ConjugateChiIndex::usage =
"ConjugateChiIndex[m,j] gives the index of conjugate Dirichlet-character, 
 with modulas m and index j."

RealDirichletCharacterQ::usage =
"RealDirichletCharacterQ[m,j] gives the test function of real 
 Dirichlet-character or complex, with modulas m and index j."

PrimitiveDirichletCharacterQ::usage =
"PrimitiveDirichletCharacterQ[m,j] gives the test function of primitive 
 Dirichlet-character or not, with modulas m and index j."

GrundzahlD::usage =
"GrundzahlD[mMin,mMax] gives the list of Grundzahl-discriminant-D 
 in integer interval mMin to mMax.
 GrundzahlD[{mMin,mMax}]=GrundzahlD[mMin,mMax], and 
 GrundzahlD[m]=GrundzahlD[m,m]."

GrundzahlDQ::usage =
"GrundzahlDQ[m] gives the test function of integer m is Grundzahl-
 discriminant-D or not."

QuadraticFieldDedekindZeta::usage =
"QuadraticFieldDedekindZeta[d,s] gives the value of the Dedekind-zeta 
 function on the quadratic field Q(Sqrt[d]).
 Here, d is integer number of square free."

CyclotomicFieldDedekindZeta::usage =
"QuadraticFieldDedekindZeta[m,s] gives the value of the Dedekind-zeta 
 function on the cyclotomic field Q(Exp[2*Pi*I*k/m]) && (m,k)=1.
 Here, m is positive odd number."

ImprovedLerchPhi::usage =
"ImprovedLerchPhi[z,s,a] gives the value of the Lerch function.
 This function, improved the same function of Mathematica."

ZetaPrime::usage =
"ZetaPrime[d,s] gives the value of the d-th derivative of Riemann's
 zeta function."

StieltjesFGamma::usage =
"StieltjesFGamma[s] gives the value of the Stieltjes function."

SitaramachandraraoDelta::usage =
"SitaramachandraraoDelta[s] gives the value of the Sitaramachandrarao 
 function."

ZetaZeta::usage =
"ZetaZeta[s] gives the value of the zeta function over nontrivial zeros of 
 Riemann's zeta functions, 
 ZetaZeta[s]=Sum[Im[ZetaZero[n]]^(-s),{n,1,Infinity}]."

LehmerZetaZeta::usage =
"LehmerZetaZeta[s] gives the value of the zeta function over nontrivial 
 zeros of Riemann's zeta functions of Lehmer-type, 
 LehmerZetaZeta[s]=Sum[ZetaZero[n]^(-s),{n,-Infinity,Infinity}]."

BellExponentB::usage =
"BellExponentB[s] gives the value of the Bell function."

BellExponentB2::usage =
"BellExponentB2[s] gives the value of the second Bell function."

PrimeNumberZetaP::usage =
"PrimeNumberZetaP[s] gives the value of the Prime zeta function.
 It's analogous to the Riemann's zeta function at 
 Sum[1/Prime[n]^s,{n,1,Infinity}].
 Branch cut lines of PrimeNumberZetaP[s] is horizontal form."

PrimeNumberZetaP2::usage =
"PrimeNumberZetaP2[s] gives the value of the Prime zeta function.
 It's analogous to the Riemann's zeta function at 
 Sum[1/Prime[n]^s,{n,1,Infinity}].
 Branch cut lines of PrimeNumberZetaP2[s] is non horizontal(curve) form."

RamanujanLTau::usage =
"RamanujanLTau[s] gives the value of the Ramanujan's L-Dirichlet series
 Sum[RamanujanTau[n]*n^(-s),{n,1,Infinity}]."

RamanujanSiegelTheta::usage =
"RamanujanSiegelTheta[t] is analogous to RiemannSiegelTheta[t] in the 
 theory of the Riemann zeta function. 
 In particular, Exp[I*RamanujanSiegelTheta[t]]*RamanujanLTau[6+I*t] is 
 RamanujanSiegelZ[t] and is a real valued function if t is real."

RamanujanSiegelZ::usage =
"RamanujanSiegelZ[t] evaluates the function
 Exp[I RamanujanSiegelTheta[t]]*RamanujanLTau[6+I*t].
 RamanujanSiegelZ[t] is analogous to RiemannSiegelZ[t] in the theory of 
 the Riemann zeta function."

EichlerEllipticCurveL::usage =
"EichlerEllipticCurveL[s] gives the value of the Eichler's elliptic curve-
 Dirichlet series Sum[c[n]*n^(-s),{n,1,Infinity}], 
 Here, c[n] is Fourier coefficient of cusp form : 
 q*(QPochhammer[q]*QPochhammer[q^11])^2 = Sum[c[n]*q^n,{n,1,Infinity}]."

EichlerCoefficientC::usage =
"EichlerCoefficientC[n] gives the value of the Fourier coefficient of cusp form : 
 q*(QPochhammer[q]*QPochhammer[q^11])^2 = Sum[c[n]*q^n,{n,1,Infinity}]."

FibonacciZeta::usage =
"FibonacciZeta[s] gives the value of the Fibonacci zeta function.
 ZF[s]=Sum[1/Fibonacci[k]^s,{k,1,Infinity}]."

RiemannPrimeR::usage =
"RiemannPrimeR[z] gives the value of the Riemann's prime-counting function
 R(z)=1+Sum[Log[z]^n/(n! n Zeta[n+1]),{n,1,Infinity}], or 
 R(z)=Sum[MoebiusMu[n] LogIntegral[z^(1/n)]/n,{n,1,Infinity}]."

RiemannPrimeRExp::usage =
"RiemannPrimeRExp[z] gives the value of the Riemann's prime-counting function
 R(Exp[z])=1+Sum[z^n/(n! n Zeta[n+1]),{n,1,Infinity}], or 
 R(Exp[z])=Sum[MoebiusMu[n] LogIntegral[Exp[z/n]]/n,{n,1,Infinity}]."

RiemannExpIntegral::usage =
"RiemannExpIntegral[z] gives the value of the Riemann's integral function in
 prime number-approximating theory, 
 ri(z)=Integrate[1/(x (Exp[2 x]-1)),{x,z,Infinity}]."

RiemannLogIntegral::usage =
"RiemannLogIntegral[z] gives the value of the Riemann's integral function in
 prime number-approximating theory, 
 Ri(z)=Integrate[1/(x (x^2-1) Log[x]),{x,z,Infinity}] 
 and rerated ri(Log[z])=Ri(z)."

RiemannVonMangoldtPi::usage =
"RiemannVonMangoldtPi[k,kMax,x] gives the value of the Riemann-von Mangoldt's 
 formula which is an approximation of prime number step function Pi(x).
 Here, k is the number of non-trivial zeros of the Riemann zeta function 
 counted from the smallest absolute value, and kMax is maximum of k.
 Assume finite addition in the series of exponential integrals."

RiemannVonMangoldtPiExact::usage =
"RiemannVonMangoldtPiExact[k,kMax,x] gives the value of the Riemann-von Mangoldt's 
 formula which is an approximation of prime number step function Pi(x).
 Here, k is the number of non-trivial zeros of the Riemann zeta function 
 counted from the smallest absolute value, and kMax is maximum of k.
 Assume exact infinite addition in the series of exponential integrals."

ChebyshevPrimeTheory::usage =
"ChebyshevPrimeTheory[k,kMax,x] gives the value of the Chebyshev's formula 
 which is an approximation of Psi(x)=ChebyshevPsi[x].
 Here, k is the number of non-trivial zeros of the Riemann zeta function 
 counted from the smallest absolute value, and kMax is maximum of k."

ChebyshevPsi::usage =
"ChebyshevPsi[x] gives the value of the Chebyshev's prime number step 
 function: Psi(x)=Sum[MangoldtLambda[n],{n,1,Floor[x]}]."

EulerSumH::usage =
"EulerSumH[s] gives the value of the Euler sums : 
 Sum[HarmonicNumber[n]*n^(-s),{n,1,Infinity}]. 
 EulerSumH[s,z] gives the value of the General Euler sums : 
 Sum[HarmonicNumber[n,z]*n^(-s),{n,1,Infinity}].
 Here, EulerSumH[s,1]=EulerSumH[s]."

RieszR::usage =
"RieszR[z] gives the value of the Riesz function : 
 RieszR[z]=Sum[-(-z)^n/((n-1)!*Zeta[2n]),{n,1,Infinity}]."

HardyLittlewoodR::usage =
"HardyLittlewoodR[z] gives the value of the Hardy-Littlewood function : 
 HardyLittlewoodR[z]=Sum[(-z)^n/(n!*Zeta[2n+1]),{n,1,Infinity}]."

ClausenCL2::usage =
"ClausenCL2[z] gives the Clausen's function.
 CL2(z)=Integrate[-Log[2 Sin[t/2]],{t,0,z}]."

PeriodicClausenCL2::usage =
"PeriodicClausenCL2[z] gives the Periodic Clausen's function.
 CLp2(z+2Pi)=CLp2(z), CLp2(z)=CL2(z)(0<=Re[z]<2Pi)."

GeneralClausenC::usage =
"GeneralClausenC[s,z] gives the Generalized Clausen's C-function.
 C(s,z)=Sum[Cos[k*z]/(k^s),{k,1,Infinity}].
 GeneralClausenC[s,z,prec] gives the GeneralClausenC[s,z] with high 
 precision s and z."

GeneralClausenS::usage =
"GeneralClausenS[s,z] gives the Generalized Clausen's S-function.
 S(s,z)=Sum[Sin[k*z]/(k^s),{k,1,Infinity}].
 GeneralClausenS[s,z,prec] gives the GeneralClausenS[s,z] with high 
 precision s and z.
 In particular, GeneralClausenS[2,z]=ClausenCL2[z]."

PeriodicGeneralClausenC::usage =
"PeriodicGeneralClausenC[s,z] gives the Periodic generalized Clausen's C-function.
 Cp(s,z+2Pi)=Cp(s,z), Cp(s,z)=C(s,z)(0<=Re[z]<2Pi).
 PeriodicGeneralClausenC[s,z,prec] gives the PeriodicGeneralClausenC[s,z] 
 with high precision s and z."

PeriodicGeneralClausenS::usage =
"PeriodicGeneralClausenS[s,z] gives the Periodic generalized Clausen's S-function.
 Sp(s,z+2Pi)=Sp(s,z), Sp(s,z)=S(s,z)(0<=Re[z]<2Pi).
 PeriodicGeneralClausenS[s,z,prec] gives the PeriodicGeneralClausenS[s,z] 
 with high precision s and z.
 In particular, PeriodicGeneralClausenS[2,z]=PeriodicClausenCL2[z]."

RogersL::usage =
"RogersL[z] gives the Rogers's double logarithms.
 L(z)=PolyLog[2,z]+Log[z]*Log[1-z]/2."

ArcTanIntegralTi::usage =
"ArcTanIntegralTi[s,z] gives the Generalized arctangent integral.
 Example : Ti(2,z)=Integrate[ArcTan[t]/t,{t,0,z}].
 ArcTanIntegralTi[s,z,prec] gives the ArcTanIntegralTi[s,z] with 
 high precision s and z."

DebyeD1::usage =
"DebyeD1[s,z] gives the Debye's function.
 D1(s,z)=Integrate[t^s/(Exp[t]-1),{t,0,z}]/Gamma[s+1].
 Type of the DebyeD1[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 DebyeD1[s,z]=DebyeD1[s,z,1]."

DebyeD2::usage =
"DebyeD2[s,z] gives the Debye's function.
 D2(s,z)=Integrate[t^s/(Exp[t]-1),{t,z,Infinity}]/Gamma[s+1].
 Type of the DebyeD2[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 DebyeD2[s,z]=DebyeD2[s,z,1]."

GrueneisenG1::usage =
"GrueneisenG1[s,z] gives the Grueneisen's function.
 G1(s,z)=Integrate[t^s*Exp[t]/(Exp[t]-1)^2,{t,0,z}]/Gamma[s+1].
 Type of the GrueneisenG1[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 GrueneisenG1[s,z]=GrueneisenG1[s,z,1]."

GrueneisenG2::usage =
"GrueneisenG2[s,z] gives the Grueneisen's function.
 G2(s,z)=Integrate[t^s*Exp[t]/(Exp[t]-1)^2,{t,z,Infinity}]/Gamma[s+1].
 Type of the GrueneisenG2[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 GrueneisenG2[s,z]=GrueneisenG2[s,z,1]."

StroemgrenPhi1::usage =
"StroemgrenPhi1[s,z] gives the Stroemgren's function.
 Phi1(s,z)=Integrate[t^s*Exp[2t]/(Exp[t]-1)^3,{t,0,z}]/Gamma[s+1].
 Type of the StroemgrenPhi1[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 StroemgrenPhi1[s,z]=StroemgrenPhi1[s,z,1]."

StroemgrenPhi2::usage =
"StroemgrenPhi2[s,z] gives the Stroemgren's function.
 Phi2(s,z)=Integrate[t^s*Exp[2t]/(Exp[t]-1)^3,{t,z,Infinity}]/Gamma[s+1].
 Type of the StroemgrenPhi2[s,z,typ] is 
 typ=1 (Left lines branch cut type) or 
 typ=2 (On imaginary axis lines branch cut type) or 
 typ=3 (Right lines branch cut type).
 StroemgrenPhi2[s,z]=StroemgrenPhi2[s,z,1]."



Begin["SpecialFunctions`Zeta`Private`"]







Attributes[RiemannXi]={Listable}

Attributes[PeriodicZetaF]={Listable}

Attributes[ImprovedHurwitzZeta]={Listable}

Attributes[ImprovedDirichletL]={Listable}

Attributes[GaussSum]={Listable}

Attributes[ConjugateChiIndex]={Listable}

Attributes[RealDirichletCharacterQ]={Listable}

Attributes[PrimitiveDirichletCharacterQ]={Listable}

Attributes[GrundzahlDQ]={Listable}

Attributes[QuadraticFieldDedekindZeta]={Listable}

Attributes[CyclotomicFieldDedekindZeta]={Listable}

Attributes[ImprovedLerchPhi]={Listable}

Attributes[ZetaPrime]={Listable}

Attributes[StieltjesFGamma]={Listable}

Attributes[SitaramachandraraoDelta]={Listable}

Attributes[ZetaZeta]={Listable}

Attributes[LehmerZetaZeta]={Listable}

Attributes[BellExponentB]={Listable}

Attributes[BellExponentB2]={Listable}

Attributes[PrimeNumberZetaP]={Listable}

Attributes[PrimeNumberZetaP2]={Listable}

Attributes[RamanujanLTau]={Listable}

Attributes[RamanujanSiegelTheta]={Listable}

Attributes[RamanujanSiegelZ]={Listable}

Attributes[EichlerEllipticCurveL]={Listable}

Attributes[EichlerCoefficientC]={Listable}

Attributes[FibonacciZeta]={Listable}

Attributes[RiemannPrimeR]={Listable}

Attributes[RiemannPrimeRExp]={Listable}

Attributes[RiemannExpIntegral]={Listable}

Attributes[RiemannLogIntegral]={Listable}

Attributes[RiemannVonMangoldtPi]={Listable}

Attributes[RiemannVonMangoldtPiExact]={Listable}

Attributes[ChebyshevPrimeTheory]={Listable}

Attributes[ChebyshevPsi]={Listable}

Attributes[EulerSumH]={Listable}

Attributes[RieszR]={Listable}

Attributes[HardyLittlewoodR]={Listable}

Attributes[ClausenCL2]={Listable}

Attributes[PeriodicClausenCL2]={Listable}

Attributes[GeneralClausenC]={Listable}

Attributes[GeneralClausenS]={Listable}

Attributes[PeriodicGeneralClausenC]={Listable}

Attributes[PeriodicGeneralClausenS]={Listable}

Attributes[RogersL]={Listable}

Attributes[ArcTanIntegralTi]={Listable}

Attributes[DebyeD1]={Listable}

Attributes[DebyeD2]={Listable}

Attributes[GrueneisenG1]={Listable}

Attributes[GrueneisenG2]={Listable}

Attributes[StroemgrenPhi1]={Listable}

Attributes[StroemgrenPhi2]={Listable}



(*****************************************************************************)
(*                               Common codes                                *)
(*****************************************************************************)

(************************* ZetaZeros lists *************************)

zetaZeroListUpper[kMax_Integer]:=
  zetaZeroListUpper[kMax]=N[ZetaZero[Range[kMax]],100];

zetaZeroListLower[kMax_Integer]:=
  zetaZeroListLower[kMax]=Conjugate[zetaZeroListUpper[kMax]];

zetaZeroList[kMax_Integer]:=
  zetaZeroList[kMax]=
  Join[zetaZeroListLower[kMax],zetaZeroListUpper[kMax]];

(**************** Zeta'[s] for internal calculations ****************)

zetaPrimeVal[s_]:=Module[{x},D[Zeta[x],x]/.(x->s)];

(************************ realORcomplex[z,f] ************************)

realORcomplex[z_,f_]:=If[Abs[Im[z]]>0,N[f],N[Re[f]]];



(*****************************************************************************)
(*                               RiemannXi[s]                                *)
(*****************************************************************************)

RiemannXi[s_]:=s*(s-1)*Pi^(-s/2)*Gamma[s/2]*Zeta[s];



(*****************************************************************************)
(*                            PeriodicZetaF[s,a]                             *)
(*****************************************************************************)

PeriodicZetaF[s_,a_]:=
  Exp[2 Pi I a]*LerchPhi[Exp[2 Pi I a],s,1];



(*****************************************************************************)
(*                         ImprovedHurwitzZeta[s,a]                          *)
(*****************************************************************************)

ImprovedHurwitzZeta[s_,a_]:=
  Zeta[s,a]/;(Precision[s]<Infinity && Not[imprHZRegionQ[s]] && 0<=Re[a]<=1)

ImprovedHurwitzZeta[s_,a_]:=
  (Exp[-Pi I (1-s)/2] PeriodicZetaF[1-s,a]+
   Exp[Pi I (1-s)/2] PeriodicZetaF[1-s,-a])*
   Gamma[1-s]/((2 Pi)^(1-s))/;
   (Precision[s]<Infinity && imprHZRegionQ[s] && 0<=Re[a]<=1)

ImprovedHurwitzZeta[s_,a_]:=
  ImprovedHurwitzZeta[s,a+1]+(-1/a)^s/;(Precision[s]<Infinity && Re[a]<0)

ImprovedHurwitzZeta[s_,a_]:=
  ImprovedHurwitzZeta[s,a-1]-(a-1)^(-s)/;(Precision[s]<Infinity && Re[a]>1)

imprHZRegionQ[s_]:=
  If[(Re[s]<=-3 && Abs[Im[s]]<15) || Abs[Arg[s+3]]>1.67 || 
      Abs[Arg[s+5.5]]>1.62,True,False];



(*****************************************************************************)
(*                           Dirichlet L-functions                           *)
(*****************************************************************************)

(* ImprovedDirichletL[m,j,s] : Dirichlet L-functions *)
ImprovedDirichletL[m_Integer,j_Integer,s_]:=
  DirichletL[m,j,s]/;
  (Re[s]>=-2 && 1<=j<=EulerPhi[m])

ImprovedDirichletL[m_Integer,j_Integer,s_]:=
  Module[{pM,pJ,a},
    {pM,pJ}=primitiveChi[m,j];
    a=(1-DirichletCharacter[pM,pJ,-1])/2;
    Sqrt[pM/Pi]*(Pi/pM)^s*Gamma[(1-s+a)/2]/Gamma[(s+a)/2]*
      DirichletL[m,j,1-s]*factorNonPrimitiveL[m,pM,pJ,s]]/;
  (Re[s]<-2 && 1<=j<=EulerPhi[m] && realChiQ[m,j])

ImprovedDirichletL[m_Integer,j_Integer,s_]:=
  Module[{pM,pJ,a},
    {pM,pJ}=primitiveChi[m,j];
    a=(1-DirichletCharacter[pM,pJ,-1])/2;
    I^(-a)/Sqrt[Pi]*GaussSum[pM,pJ]*(Pi/pM)^s*Gamma[(1-s+a)/2]/Gamma[(s+a)/2]*
      DirichletL[m,conjChiIndex[m,j],1-s]*factorNonPrimitiveL[m,pM,pJ,s]]/;
  (Re[s]<-2 && 1<=j<=EulerPhi[m] && Not[realChiQ[m,j]])

factorNonPrimitiveL[m_,pM_,pJ_,s_]:=
  Apply[Times,
    Map[(1-DirichletCharacter[pM,pJ,#]*#^(-s))/
      (1-Conjugate[DirichletCharacter[pM,pJ,#]]*#^(s-1))&,
      Select[Divisors[m],PrimeQ]]];

(* GaussSum[m,j] : Gauss's sum *)
GaussSum[m_Integer,j_Integer]:=
  Sum[DirichletCharacter[m,j,n]*Exp[2 Pi*I*n/m],{n,1,m}]/;
  (1<=j<=EulerPhi[m])

(* ConjugateChiIndex[m,j] : Index of conjugate character *)
conjChiIndex[m_,j_]:=
  conjChiIndex[m,j]=ConjugateChiIndex[m,j];

ConjugateChiIndex[m_Integer,j_Integer]:=
  Module[{allChi,conjChi},
    allChi=Table[DirichletCharacter[m,i,n],{i,1,EulerPhi[m]},{n,0,m-1}];
    conjChi=Conjugate[Table[DirichletCharacter[m,j,n],{n,0,m-1}]];
    Position[allChi,conjChi][[1,1]]]/;
  (1<=j<=EulerPhi[m])

(* RealDirichletCharacterQ[m,j] : Test function of Real character *)
realChiQ[m_,j_]:=
  realChiQ[m,j]=realDirichletChiQ[m,j];

realDirichletChiQ[m_Integer,j_Integer]:=
  Module[{chi},
    chi=Table[DirichletCharacter[m,j,n],{n,0,m-1}];
    FreeQ[Map[#===0&,Im[N[chi]]],False]];

RealDirichletCharacterQ[m_Integer,j_Integer]:=
  If[realDirichletChiQ[m,j],
    Row[{"\[Chi]",{m,j},"(n) is real Dirichlet-character."}],
    Row[{"\[Chi]",{m,j},"(n) is complex Dirichlet-character."}]]/;
  (1<=j<=EulerPhi[m])

(* PrimitiveDirichletCharacterQ[m,j] : Test function of primitive character *)
primitiveChi[m_Integer,j_Integer]:=
  primitiveChi[m,j]=primitiveDirichletChi[m,j];

primitiveDirichletChi[m_Integer,j_Integer]:=
  Module[{chi,divM,n01,priMJ,testChi,priJ},
    chi=Table[DirichletCharacter[m,j,n],{n,m}];
    divM=Rest[Most[Divisors[m]]];
    n01=Table[If[GCD[n,m]>1,0,1],{n,m}];
    priMJ={};
    While[(priMJ==={} && divM=!={} && j=!=1),
      testChi=Map[PadRight[#,m,#]*n01&,
        Table[DirichletCharacter[divM[[1]],i,n],{i,EulerPhi[divM[[1]]]},{n,divM[[1]]}]];
      priJ=Position[testChi,chi];
      priMJ=If[priJ==={},priMJ,{divM[[1]],priJ[[1,1]]}];
      divM=Rest[divM]];
    If[j===1,{1,1},If[priMJ==={},{m,j},priMJ]]];

PrimitiveDirichletCharacterQ[m_Integer,j_Integer]:=
  Module[{priMJ=primitiveDirichletChi[m,j]},
    If[priMJ==={m,j},
      Row[{"\[Chi]",{m,j},"(n) is primitive Dirichlet-character."}],
      Row[{"\[Chi]",{m,j},"(n) is not primitive Dirichlet-character.
More primitive Dirichlet-character is \[Chi]",priMJ,"(n)."}]]]/;
  (1<=j<=EulerPhi[m])

(* GrundzahlDQ[m] : Test function of "Grundzahl" discriminant-D *)
GrundzahlDQ[m_Integer]:=
  If[(Mod[m,4]===1 && SquareFreeQ[m]) || 
    (Mod[m,4]===0 && SquareFreeQ[m/4] && MemberQ[{2,3},Mod[m/4,4]]),
  True,False];

(* GrundzahlD[mMin,mMax] : List of "Grundzahl" discriminant-D *)
GrundzahlD[mMin_Integer,mMax_Integer]:=
  Select[Table[m,{m,mMin,mMax,1}],GrundzahlDQ[#]&];

GrundzahlD[m_Integer]:=GrundzahlD[m,m];

GrundzahlD[mL_List]:=Apply[GrundzahlD,mL]/;(Length[mL]===2)



(*****************************************************************************)
(*                      QuadraticFieldDedekindZeta[d,s]                      *)
(*****************************************************************************)

QuadraticFieldDedekindZeta[d_Integer,s_]:=
  Zeta[s]*ImprovedDirichletL[Abs[gruD[d]],quadraticChiIndexJ[d],s]/;
  SquareFreeQ[d]

quadraticChiIndexJ[d_]:=
  quadraticChiIndexJ[d]=
  Module[{absGD,chiL,chi},
    absGD=Abs[gruD[d]];
    chiL=Table[DirichletCharacter[absGD,j,n],{j,1,EulerPhi[absGD]},{n,0,absGD-1}];
    chi=Table[JacobiSymbol[gruD[d],n],{n,0,absGD-1}];
    Position[chiL,chi][[1,1]]];

gruD[d_]:=
  Which[Mod[d,4]===1,d,MemberQ[{2,3},Mod[d,4]],4*d];



(*****************************************************************************)
(*                     CyclotomicFieldDedekindZeta[m,s]                      *)
(*****************************************************************************)

CyclotomicFieldDedekindZeta[m_,s_]:=
  Zeta[s]*
  Apply[Times,Map[ImprovedDirichletL[#[[1]],#[[2]],s]&,allPrimitiveChi[m]]];

allPrimitiveChi[m_Integer]:=
  allPrimitiveChi[m]=
  Select[Table[primitiveDirichletChi[m,j],{j,EulerPhi[m]}],#[[1]]===m&];



(*****************************************************************************)
(*                          ImprovedLerchPhi[z,s,a]                          *)
(*****************************************************************************)

ImprovedLerchPhi[z_,s_,a_]:=
  If[Chop[Abs[Im[z]]]===0 && Re[z]<=1 && Chop[Abs[Im[s]]]===0 && Chop[Abs[Im[a]]]===0,
  Re[imprLerchPhi[z,s,a]],imprLerchPhi[z,s,a]];

imprLerchPhi[z_,s_,a_]:=PolyLog[s,z]/;(N[Re[a]]===0.)

imprLerchPhi[z_,s_,a_]:=
  Module[{nMax=10,kMax=15},
    z^nMax*(nMax+a)^(-s)/2+
    z^(-a)*(nMax+a)^(1-s)*ExpIntegralE[s,-(nMax+a) Log[z]]+
    Sum[z^n*(n+a)^(-s),{n,0,nMax-1}]-
    Sum[BernoulliB[2 j]/((2 j)!) derTermLerch[{z,s,a},nMax,2 j-1],{j,1,kMax}]]/;
  (Re[s]>=-1 && 0<Re[a]<=1)

derTermLerch[{z_,s_,a_},x_,j_]:=
  z^x*Sum[(-1)^n*Binomial[j,n] Log[z]^(j-n) Pochhammer[s,n] (x+a)^(-n-s),{n,0,j}];

imprLerchPhi[z_,s_,a_]:=
  Module[{sum=0,sumO=-1,k=1,ef=Exp[2 Pi*I*a]},
    While[(sumO!=sum && k<200),sumO=sum;
      sum+=((2 Pi*I*k-Log[z])^(s-1)*ef+(-2 Pi*I*k-Log[z])^(s-1)/ef);
      ef*=Exp[2 Pi*I*a];k++];
    z^(-a)*Gamma[1-s] ((-Log[z])^(s-1)+sum)]/;
  (Re[s]<-1 && 0<Re[a]<=1 && Re[z]>=0)

imprLerchPhi[z_,s_,a_]:=
  Module[{sum=0,sumO=-1,k=1,ep=Exp[3 Pi*I*a],em=Exp[-Pi*I*a]},
    While[(sumO!=sum && k<200),sumO=sum;
      sum+=(((2 k+1) Pi*I-Log[-z])^(s-1)*ep+((-2 k+1) Pi*I-Log[-z])^(s-1)*em);
      ep*=Exp[2 Pi*I*a];em*=Exp[-2 Pi*I*a];k++];
    (-z)^(-a)*Gamma[1-s] ((Pi*I-Log[-z])^(s-1)*Exp[Pi*I*a]+sum)]/;
  (Re[s]<-1 && 0<Re[a]<=1 && Re[z]<0)

imprLerchPhi[z_,s_,a_]:=
  Module[{frcA=Re[a]-Floor[Re[a]]+Im[a]*I,intA=Floor[Re[a]]},
    z^(-intA)*(imprLerchPhi[z,s,frcA]-
    Sum[z^k*((k+frcA)^2)^(-s/2),{k,0,intA-1}])]/;
  (Re[a]>1)

imprLerchPhi[z_,s_,a_]:=
  Module[{frcA=Re[a]-Floor[Re[a]]+Im[a]*I,intA=Floor[Re[a]]},
    z^(-intA)*imprLerchPhi[z,s,frcA]+
    Sum[z^k*((k+frcA+intA)^2)^(-s/2),{k,0,-intA-1}]]/;
  (Re[a]<0)



(*****************************************************************************)
(*                     ImprovedLerchPhi[z,s,a,DoublySum]                     *)
(*****************************************************************************)

ImprovedLerchPhi[z_,s_,a_,"DoublySum"]:=
  ImprovedLerchPhi[z,s,a]+ImprovedLerchPhi[1/z,s,-a]-(a^2)^(-s/2);



(*****************************************************************************)
(*                              ZetaPrime[d,s]                               *)
(*****************************************************************************)

ZetaPrime[d_,s_]:=Zeta[s]/;(N[Abs[d]]===0.)

ZetaPrime[d_,s_]:=Infinity/;(N[Abs[d]]=!=0. && N[s]===1.)

ZetaPrime[d_,s_]:=
  realORcomplex[s,Module[{m=20,sdtm=5},
     (Log[m]^d)/(m^s)/2+zetaPrIntegTerm[d,s,m]+
     Sum[(Log[i]^d)/(i^s),{i,2,m-1,1}]-
     Sum[zetaPrDTerm[d,s,m,j-1]BernoulliB[j]/(j!),{j,2,2*sdtm,2}]]]*(-1)^d/;
     (N[Abs[d]]=!=0. && Precision[s]<Infinity && Re[s]>=0.31 && N[s]=!=1.)

zetaPrIntegTerm[d_,s_,m_]:=
  Fold[(Pochhammer[d+1-#2,#2]+(s-1) Log[m] #1)&,1,Range[1,d,1]]*
  m^(1-s)/((s-1)^(d+1));

zetaPrDTerm[d_,s_,m_,k_]:=
  Module[{x},D[(Log[x]^d)/(x^s),{x,k}]/.(x->m)]/;(k>=11)

zetaPrDTerm[d_,s_,m_,1]:=
  m^(-1-s) Log[m]^(-1+d) (d-s Log[m]);

zetaPrDTerm[d_,s_,m_,3]:=
  m^(-3-s) Log[m]^(-3+d) (d (2-3 d+d^2)-3 (-1+d) d (1+s) Log[m]+
  d (2+6 s+3 s^2) Log[m]^2-s (2+3 s+s^2) Log[m]^3);

zetaPrDTerm[d_,s_,m_,5]:=
  m^(-5-s) Log[m]^(-5+d) (d (24-50 d+35 d^2-10 d^3+d^4)-
  5 d (-6+11 d-6 d^2+d^3) (2+s) Log[m]+
  5 d (2-3 d+d^2) (7+8 s+2 s^2) Log[m]^2-
  5 (-1+d) d (10+21 s+12 s^2+2 s^3) Log[m]^3+
  d (24+100 s+105 s^2+40 s^3+5 s^4) Log[m]^4-
  s (24+50 s+35 s^2+10 s^3+s^4) Log[m]^5);

zetaPrDTerm[d_,s_,m_,7]:=
  m^(-7-s) Log[m]^(-7+d) (d (720-1764 d+1624 d^2-735 d^3+175 d^4-21 d^5+d^6)-
  7 d (-120+274 d-225 d^2+85 d^3-15 d^4+d^5) (3+s) Log[m]+
  7 d (24-50 d+35 d^2-10 d^3+d^4) (25+18 s+3 s^2) Log[m]^2-
  35 d (-6+11 d-6 d^2+d^3) (21+25 s+9 s^2+s^3) Log[m]^3+
  7 d (2-3 d+d^2) (232+420 s+250 s^2+60 s^3+5 s^4) Log[m]^4-
  7 (-1+d) d (252+696 s+630 s^2+250 s^3+45 s^4+3 s^5) Log[m]^5+
  d (720+3528 s+4872 s^2+2940 s^3+875 s^4+126 s^5+7 s^6) Log[m]^6-
  s (720+1764 s+1624 s^2+735 s^3+175 s^4+21 s^5+s^6) Log[m]^7);

zetaPrDTerm[d_,s_,m_,9]:=
  m^(-9-s) Log[m]^(-9+d)*
  (d (40320-109584 d+118124 d^2-67284 d^3+22449 d^4-4536 d^5+546 d^6-36 d^7+d^8)-
  9 d (-5040+13068 d-13132 d^2+6769 d^3-1960 d^4+322 d^5-28 d^6+d^7) (4+s) Log[m]+
  6 d (720-1764 d+1624 d^2-735 d^3+175 d^4-21 d^5+d^6) (91+48 s+6 s^2) Log[m]^2-
  42 d (-120+274 d-225 d^2+85 d^3-15 d^4+d^5) (108+91 s+24 s^2+2 s^3) Log[m]^3+
  21 d (24-50 d+35 d^2-10 d^3+d^4) (1069+1296 s+546 s^2+96 s^3+6 s^4) Log[m]^4-
  21 d (-6+11 d-6 d^2+d^3) (3204+5345 s+3240 s^2+910 s^3+120 s^4+6 s^5) Log[m]^5+
  2 d (2-3 d+d^2) (59062+134568 s+112245 s^2+45360 s^3+9555 s^4+1008 s^5+42 s^6) Log[m]^6-
  6 (-1+d) d (18264+59062 s+67284 s^2+37415 s^3+11340 s^4+1911 s^5+168 s^6+6 s^7) Log[m]^7+
  3 d (13440+73056 s+118124 s^2+89712 s^3+37415 s^4+9072 s^5+1274 s^6+96 s^7+3 s^8) Log[m]^8-
  s (40320+109584 s+118124 s^2+67284 s^3+22449 s^4+4536 s^5+546 s^6+36 s^7+s^8) Log[m]^9);

ZetaPrime[d_,s_]:=
  realORcomplex[s,Module[{m=3,sdtm=20},
     (Log[m]^d)/(m^s)/2+zetaPrIntegTerm[d,s,m]+
     Sum[(Log[i]^d)/(i^s),{i,2,m-1,1}]-
     Sum[zetaPrDTerm[d,s,m,j-1]BernoulliB[j]/(j!),{j,2,2*sdtm,2}]]]*(-1)^d/;
     (N[Abs[d]]=!=0. && Precision[s]<Infinity && nearlyEvenQ[s]===True && Re[s]<0.31)

ZetaPrime[d_,s_]:=
  zetaPrimeNegative[d,s]/;
  (N[Abs[d]]=!=0. && Precision[s]<Infinity && nearlyEvenQ[s]=!=True && Re[s]<0.31)

nearlyEvenQ[v_]:=
  EvenQ[Chop[IntegerPart[Re[v]]]+Chop[FractionalPart[Re[v]]]+Chop[Im[v]]*I];

zetaPrimeNegative[d_,s_]:=
  Module[{x},
  D[Pi^(x-1/2) Gamma[(1-x)/2]/Gamma[x/2] Zeta[1-x],{x,d}]/.
  Derivative[d][Zeta][1-x]->ZetaPrime[d,1-x]/.x->s]/;(d>4)

zetaPrimeNegative[1,s_]:=
  (Pi^(s-1/2) Gamma[(1-s)/2]*
  ((2 Log[Pi]-PolyGamma[0,(1-s)/2]-PolyGamma[0,s/2])*
  Zeta[1-s]-2 ZetaPrime[1,1-s]))/(2 Gamma[s/2])

zetaPrimeNegative[2,s_]:=
  (Pi^(s-1/2) Gamma[(1-s)/2]*
  ((4 Log[Pi]^2-(4 Log[Pi]-PolyGamma[0,(1-s)/2]-PolyGamma[0,s/2])*
  (PolyGamma[0,(1-s)/2]+PolyGamma[0,s/2])+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])*
  Zeta[1-s]+4 (-2 Log[Pi]+PolyGamma[0,(1-s)/2]+PolyGamma[0,s/2])*
  ZetaPrime[1,1-s]+4 ZetaPrime[2,1-s]))/(4 Gamma[s/2])

zetaPrimeNegative[3,s_]:=
  (Pi^(s-1/2) Gamma[(1-s)/2]*
  ((8 Log[Pi]^3-PolyGamma[0,(1-s)/2]^3+PolyGamma[0,(1-s)/2]^2*
  (6 Log[Pi]-3 PolyGamma[0,s/2])+6 Log[Pi] PolyGamma[0,s/2]^2-
  PolyGamma[0,s/2]^3+6 Log[Pi] (PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])-
  3 PolyGamma[0,s/2] (4 Log[Pi]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])-
  3 PolyGamma[0,(1-s)/2] (4 Log[Pi]^2-4 Log[Pi] PolyGamma[0,s/2]+
  PolyGamma[0,s/2]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])-
  PolyGamma[2,(1-s)/2]-PolyGamma[2,s/2]) Zeta[1-s]-
  6 (4 Log[Pi]^2-(4 Log[Pi]-PolyGamma[0,(1-s)/2]-PolyGamma[0,s/2])*
  (PolyGamma[0,(1-s)/2]+PolyGamma[0,s/2])+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])*
  ZetaPrime[1,1-s]+12 (2 Log[Pi]-PolyGamma[0,(1-s)/2]-PolyGamma[0,s/2])*
  ZetaPrime[2,1-s]-8 ZetaPrime[3,1-s]))/(8 Gamma[s/2])

zetaPrimeNegative[4,s_]:=
  (Pi^(s-1/2) Gamma[(1-s)/2]*
  ((16 Log[Pi]^4+PolyGamma[0,(1-s)/2]^4-8 Log[Pi] PolyGamma[0,s/2]^3+
  PolyGamma[0,s/2]^4+4 PolyGamma[0,(1-s)/2]^3 (-2 Log[Pi]+PolyGamma[0,s/2])+
  24 Log[Pi]^2 PolyGamma[1,(1-s)/2]+3 PolyGamma[1,(1-s)/2]^2+
  6 PolyGamma[0,s/2]^2 (4 Log[Pi]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])+
  6 PolyGamma[0,(1-s)/2]^2 (4 Log[Pi]^2-4 Log[Pi] PolyGamma[0,s/2]+
  PolyGamma[0,s/2]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])-
  24 Log[Pi]^2 PolyGamma[1,s/2]-6 PolyGamma[1,(1-s)/2] PolyGamma[1,s/2]+
  3 PolyGamma[1,s/2]^2-8 Log[Pi] PolyGamma[2,(1-s)/2]-
  8 Log[Pi] PolyGamma[2,s/2]+4 PolyGamma[0,s/2]*
  (-8 Log[Pi]^3+6 Log[Pi] (-PolyGamma[1,(1-s)/2]+PolyGamma[1,s/2])+
  PolyGamma[2,(1-s)/2]+PolyGamma[2,s/2])+4 PolyGamma[0,(1-s)/2]*
  (-8 Log[Pi]^3-6 Log[Pi] PolyGamma[0,s/2]^2+
  PolyGamma[0,s/2]^3+3 PolyGamma[0,s/2]*
  (4 Log[Pi]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])+
  6 Log[Pi] (-PolyGamma[1,(1-s)/2]+PolyGamma[1,s/2])+
  PolyGamma[2,(1-s)/2]+PolyGamma[2,s/2])+PolyGamma[3,(1-s)/2]-PolyGamma[3,s/2])*
  Zeta[1-s]+8 ((-8 Log[Pi]^3+PolyGamma[0,(1-s)/2]^3-6 Log[Pi] PolyGamma[0,s/2]^2+
  PolyGamma[0,s/2]^3+3 PolyGamma[0,(1-s)/2]^2 (-2 Log[Pi]+PolyGamma[0,s/2])+
  3 PolyGamma[0,s/2] (4 Log[Pi]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])+
  3 PolyGamma[0,(1-s)/2] (4 Log[Pi]^2-4 Log[Pi] PolyGamma[0,s/2]+
  PolyGamma[0,s/2]^2+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])+
  6 Log[Pi] (-PolyGamma[1,(1-s)/2]+PolyGamma[1,s/2])+
  PolyGamma[2,(1-s)/2]+PolyGamma[2,s/2]) ZetaPrime[1,1-s]+
  3 (4 Log[Pi]^2-(4 Log[Pi]-PolyGamma[0,(1-s)/2]-PolyGamma[0,s/2])*
  (PolyGamma[0,(1-s)/2]+PolyGamma[0,s/2])+PolyGamma[1,(1-s)/2]-PolyGamma[1,s/2])*
  ZetaPrime[2,1-s]+4 (-2 Log[Pi]+PolyGamma[0,(1-s)/2]+PolyGamma[0,s/2])*
  ZetaPrime[3,1-s])+16 ZetaPrime[4,1-s]))/(16 Gamma[s/2])



(*****************************************************************************)
(*                            StieltjesFGamma[s]                             *)
(*****************************************************************************)

setPrecSMax[x_]:=
  Floor[12.485+3.69127*x+0.004886*x^2];

sdTermMax[x_]:=
  Floor[2.06+0.11*x+0.002*x^2];

maxSFRegion[z_]:=
  If[N[Abs[z]]===0.,0,If[Abs[Arg[z]]<N[ArcTan[3]],Re[z],Abs[Im[z]/3]]];

setPrecStieltjes[z_]:=
  Module[{spsm=setPrecSMax[maxSFRegion[z]]},
    SetPrecision[z*10^spsm,spsm]/(10^spsm)];

StieltjesFGamma[z_]:=Infinity/;(N[z]===-1.)

StieltjesFGamma[0]:=EulerGamma-1;

StieltjesFGamma[z_]:=
  realORcomplex[z,
  Module[{s=setPrecStieltjes[z],m=20,sdtm},
    sdtm=sdTermMax[maxSFRegion[s]];
      Log[m]^s/(2 m)-Log[m]^(1+s)/(1+s)+
      Sum[Log[i]^s/i,{i,2,m-1,1}]-
      Sum[stieltjesDTerm[s,m,j-1]*BernoulliB[j]/(j!),{j,2,2*sdtm,2}]]]/;
  (Precision[z]<Infinity && N[z]=!=-1.)

stieltjesDTerm[s_,m_,k_]:=
  Module[{x},D[Log[x]^s/x,{x,k}]/.(x->m)]/;(k>=21)

stieltjesDTerm[s_,m_,1]:=
  (s-Log[m])*Log[m]^(s-1)/(m^2);

stieltjesDTerm[s_,m_,3]:=
  (s (2-3 s+s^2)-6 (-1+s) s Log[m]+11 s Log[m]^2-6 Log[m]^3)*
  Log[m]^(s-3)/(m^4);

stieltjesDTerm[s_,m_,5]:=
  (s (24-50 s+35 s^2-10 s^3+s^4)-15 s (-6+11 s-6 s^2+s^3) Log[m]+
  85 s (2-3 s+s^2) Log[m]^2-225 (-1+s) s Log[m]^3+
  274 s Log[m]^4-120 Log[m]^5)*
  Log[m]^(s-5)/(m^6);

stieltjesDTerm[s_,m_,7]:=
  (s (720-1764 s+1624 s^2-735 s^3+175 s^4-21 s^5+s^6)-
  28 s (-120+274 s-225 s^2+85 s^3-15 s^4+s^5) Log[m]+
  322 s (24-50 s+35 s^2-10 s^3+s^4) Log[m]^2-
  1960 s (-6+11 s-6 s^2+s^3) Log[m]^3+
  6769 s (2-3 s+s^2) Log[m]^4-
  13132 (-1+s) s Log[m]^5+13068 s Log[m]^6-5040 Log[m]^7)*
  Log[m]^(s-7)/(m^8);

stieltjesDTerm[s_,m_,9]:=
(s (40320-109584 s+118124 s^2-67284 s^3+22449 s^4-4536 s^5+546 s^6-36 s^7+s^8)-
  45 s (-5040+13068 s-13132 s^2+6769 s^3-1960 s^4+322 s^5-28 s^6+s^7) Log[m]+
  870 s (720-1764 s+1624 s^2-735 s^3+175 s^4-21 s^5+s^6) Log[m]^2-
  9450 s (-120+274 s-225 s^2+85 s^3-15 s^4+s^5) Log[m]^3+
  63273 s (24-50 s+35 s^2-10 s^3+s^4) Log[m]^4-
  269325 s (-6+11 s-6 s^2+s^3) Log[m]^5+
  723680 s (2-3 s+s^2) Log[m]^6-
  1172700 (-1+s) s Log[m]^7+1026576 s Log[m]^8-362880 Log[m]^9)*
  Log[m]^(s-9)/(m^10);

stieltjesDTerm[s_,m_,11]:=
  m^(-12)*Log[m]^(s-11)*
  (Apply[Times,s-Range[0,10]]+Log[m] (-66 Apply[Times,s-Range[0,9]]+
  Log[m] (1925 Apply[Times,s-Range[0,8]]+Log[m] (-32670 Apply[Times,s-Range[0,7]]+
  Log[m] (357423 Apply[Times,s-Range[0,6]]+
  Log[m] (-2637558 Apply[Times,s-Range[0,5]]+
  Log[m] (13339535 Apply[Times,s-Range[0,4]]+
  2 Log[m] (-22997865 Apply[Times,s-Range[0,3]]+
  2 Log[m] (26314519 Apply[Times,s-Range[0,2]]+
  18 Log[m] (-2096083 Apply[Times,s-Range[0,1]]+
  20 (83711 s-27720 Log[m]) Log[m]))))))))));

stieltjesDTerm[s_,m_,13]:=
  m^(-14)*Log[m]^(s-13)*
  (Apply[Times,s-Range[0,12]]+Log[m] (-91 Apply[Times,s-Range[0,11]]+
  Log[m] (3731 Apply[Times,s-Range[0,10]]+
  Log[m] (-91091 Apply[Times,s-Range[0,9]]+
  Log[m] (1474473 Apply[Times,s-Range[0,8]]+
  Log[m] (-16669653 Apply[Times,s-Range[0,7]]+
  Log[m] (135036473 Apply[Times,s-Range[0,6]]+
  Log[m] (-790943153 Apply[Times,s-Range[0,5]]+
  2 Log[m] (1668059393 Apply[Times,s-Range[0,4]]+
  2 Log[m] (-2489425939 Apply[Times,s-Range[0,3]]+
  6 Log[m] (846406379 Apply[Times,s-Range[0,2]]+
  48 Log[m] (-23087428 Apply[Times,s-Range[0,1]]+
  17189895 s Log[m]-5405400 Log[m]^2))))))))))));

stieltjesDTerm[s_,m_,15]:=
  m^(-16)*Log[m]^(s-15)*
  (Apply[Times,s-Range[0,14]]+Log[m] (-120 Apply[Times,s-Range[0,13]]+
  Log[m] (6580 Apply[Times,s-Range[0,12]]+
  Log[m] (-218400 Apply[Times,s-Range[0,11]]+
  Log[m] (4899622 Apply[Times,s-Range[0,10]]+
  Log[m] (-78558480 Apply[Times,s-Range[0,9]]+
  Log[m] (928095740 Apply[Times,s-Range[0,8]]+
  Log[m] (-8207628000 Apply[Times,s-Range[0,7]]+
  Log[m] (54631129553 Apply[Times,s-Range[0,6]]+
  8 Log[m] (-34100401335 Apply[Times,s-Range[0,5]]+
  Log[m] (126209013385 Apply[Times,s-Range[0,4]]+
  18 Log[m] (-18797314900 Apply[Times,s-Range[0,3]]+
  Log[m] (35118025721 Apply[Times,s-Range[0,2]]+
  360 Log[m] (-118939383 Apply[Times,s-Range[0,1]]+
  70 (1195757 s-360360 Log[m]) Log[m]))))))))))))));

stieltjesDTerm[s_,m_,17]:=
  m^(-18)*Log[m]^(s-17)*
  (Apply[Times,s-Range[0,16]]+Log[m] (-153 Apply[Times,s-Range[0,15]]+
  Log[m] (10812 Apply[Times,s-Range[0,14]]+
  Log[m] (-468180 Apply[Times,s-Range[0,13]]+
  Log[m] (13896582 Apply[Times,s-Range[0,12]]+
  Log[m] (-299650806 Apply[Times,s-Range[0,11]]+
  Log[m] (4853222764 Apply[Times,s-Range[0,10]]+
  Log[m] (-60202693980 Apply[Times,s-Range[0,9]]+
  Log[m] (577924894833 Apply[Times,s-Range[0,8]]+
  Log[m] (-4308105301929 Apply[Times,s-Range[0,7]]+
  8 Log[m] (3108980662242 Apply[Times,s-Range[0,6]]+
  Log[m] (-13778558273025 Apply[Times,s-Range[0,5]]+
  2 Log[m] (23063290577149 Apply[Times,s-Range[0,4]]+
  9 Log[m] (-6314582679473 Apply[Times,s-Range[0,3]]+
  8 Log[m] (1374404492819 Apply[Times,s-Range[0,2]]+
  90 Log[m] (-17569468023 Apply[Times,s-Range[0,1]]+
  280 (42142223 s-12252240 Log[m]) Log[m]))))))))))))))));

stieltjesDTerm[s_,m_,19]:=
  m^(-20)*Log[m]^(s-19)*
  (Apply[Times,s-Range[0,18]]+Log[m] (-190 Apply[Times,s-Range[0,17]]+
  Log[m] (16815 Apply[Times,s-Range[0,16]]+
  Log[m] (-920550 Apply[Times,s-Range[0,15]]+
  Log[m] (34916946 Apply[Times,s-Range[0,14]]+
  Log[m] (-973941900 Apply[Times,s-Range[0,13]]+
  Log[m] (20692933630 Apply[Times,s-Range[0,12]]+
  Log[m] (-342252511900 Apply[Times,s-Range[0,11]]+
  Log[m] (4465226757381 Apply[Times,s-Range[0,10]]+
  Log[m] (-46280647751910 Apply[Times,s-Range[0,9]]+
  Log[m] (381922055502195 Apply[Times,s-Range[0,8]]+
  2 Log[m] (-1251929377733775 Apply[Times,s-Range[0,7]]+
  4 Log[m] (1619204623742987 Apply[Times,s-Range[0,6]]+
  2 Log[m] (-3266306460157045 Apply[Times,s-Range[0,5]]+
  9 Log[m] (1121039837014715 Apply[Times,s-Range[0,4]]-
  2579061023230750 Apply[Times,s-Range[0,3]] Log[m]+
  4236917192642304 Apply[Times,s-Range[0,2]] Log[m]^2-
  4643123127369120 Apply[Times,s-Range[0,1]] Log[m]^3+
  2996980186233600 s Log[m]^4-844757641728000 Log[m]^5)))))))))))))));



(*****************************************************************************)
(*                        SitaramachandraraoDelta[s]                         *)
(*****************************************************************************)

SitaramachandraraoDelta[s_]:=
  Module[{m=20,sdtm=8,sp=SetPrecision[s,100],ans},
    ans=(Log[m]^sp)/2+
      Gamma[sp+1]*Exp[-Pi*I*sp]*GammaRegularized[sp+1,0,-Log[m]]+
      Sum[Log[i]^sp,{i,2,m-1}]-
      sp*(Log[m]^sp)*Sum[zetaPDerivTerm[sp,m,k-1]*BernoulliB[k]/(k!),{k,2,2*sdtm,2}];
    N[ans]]/;
  (Precision[s]<Infinity)

zetaPDerivTerm[s_,m_,k_]:=
  Sum[(-1)^i*symmetryExprSI[k-1,i]*Log[m]^i*Product[s-j,{j,1,k-i-1}],{i,0,k-1}]*
  (m*Log[m])^(-k);

symmetryExprSI[m_,n_]:=
  symmetryExprSI[m,n]=symmExprI[m,n];

symmExprI[m_,n_]:=
  m*symmExprI[m-1,n-1]+symmExprI[m-1,n]/;(0<n<m)

symmExprI[m_,n_]:=1/;(n===0)

symmExprI[m_,n_]:=m!/;(n===m)



(*****************************************************************************)
(*                                ZetaZeta[s]                                *)
(*****************************************************************************)

(***** Re[s]>=0.3 *****)
ZetaZeta[s_]:=
  Module[{nMax=3000,maxZZL,mostZZL,LastZZL,ans},
    maxZZL=Im[zetaZeroListUpper[nMax]];
    mostZZL=Most[maxZZL];
    LastZZL=Last[maxZZL];
    ans=Apply[Plus,Power[mostZZL,-s]]+1/2*LastZZL^-s+
        LastZZL^(1-s)/(2 Pi (s-1))*(1/(s-1)+Log[LastZZL/(2 Pi)]);
    N[ans]]/;
  (Re[s]>=0.3)

(***** Re[s]<0.3 *****)
ZetaZeta[s_]:=
  Module[{nMax=1000,prec=50,sp,sumTermZ,ans},
    sp=SetPrecision[s,prec];
    sumTermZ=mLambdaSqList[nMax].(logListZZ[nMax]^(s-1));
    ans=((integTermZZ[Log[nMax],sp]-sumTermZ)/Gamma[s]-Zeta[s,5/4]*2^-s)/Cos[Pi/2*s]/2;
    Chop[N[ans]]]/;
  (Re[s]<0.3 && N[Mod[s,2]]=!=0.)

mLambdaSqList[n_]:=
  mLambdaSqList[n]=MangoldtLambda[Range[2,n]]/Sqrt[Range[2,n]];

logListZZ[n_]:=
  logListZZ[n]=Log[Range[2,n]];

integTermZZ[t_,s_]:=
  2^s*(-t)^(-s)*t^s*(Gamma[s]-Gamma[s,-t/2]);

(***** s=-2n *****)
ZetaZeta[s_]:=
  (-1)^(s/2)*2^s*(1-EulerE[IntegerPart[-s]]/8)/;
  (Re[s]<0.5 && N[Mod[s,2]]===0.)



(*****************************************************************************)
(*                             LehmerZetaZeta[s]                             *)
(*****************************************************************************)

(***** Re(s)>=3 *****)
LehmerZetaZeta[s_]:=
  Module[{summ},
    summ=Total[zetaZeroList[10000]^(-s)];
    If[Abs[Im[s]]>0,summ,Re[summ]]]/;
  (Re[s]>=3 && Precision[s]<Infinity)

(***** Re(s)<-2 *****)
LehmerZetaZeta[s_]:=
  Module[{nMax=10000,sTerm},
    sTerm=mLambdaList[nMax].(Log[Range[2,nMax]]^(s-1)/Range[2,nMax]);
    -Zeta[s,3/2]/(2^s)-sTerm/Gamma[s]]/;
  (Re[s]<-2 && Precision[s]<Infinity)

mLambdaList[nMax_]:=
  mLambdaList[nMax]=MangoldtLambda[Range[2,nMax]];

(***** -2<=Re(s)<3 *****)
LehmerZetaZeta[s_]:=
  Sum[(-1)^n/(2^(n-1)*n!)*Cos[Pi (s+n)/2]*ZetaZeta[s+n]*Product[s+k-1,{k,1,n}],{n,0,10}]/;
  (-2<=Re[s]<3 && Precision[s]<Infinity)



(*****************************************************************************)
(*                    BellExponentB[s], BellExponentB2[s]                    *)
(*****************************************************************************)

BellExponentB[z_]:=
  Module[{s=0,so=1,k=0,faf},faf=1;
   While[so!=s,so=s; s+=((k+1)^z/faf);
   faf*=(k+1);k++];s]/E/;(Precision[z]<Infinity)

BellExponentB2[z_]:=
  Module[{s=0,so=1,k=0,mf,faf},mf=1;faf=1;
   While[so!=s,so=s; s+=(mf/faf*(k+1)^z);
   mf*=(-1);faf*=(k+1);k++];s]/E/;(Precision[z]<Infinity)



(*****************************************************************************)
(*                            Prime zeta function                            *)
(*****************************************************************************)

(********** Prime zeta function : Horizontal branch cut lines **********)

PrimeNumberZetaP[s_]:=
  compilePrimeZeta[s+0.0 I]/;(Re[s]>0 && Precision[s]<Infinity)

compilePrimeZeta=Compile[{{s,_Complex}},
  Module[{sum=0+0.0 I,sumO=1+0.0 I,n=1,mu},
    While[(Abs[sumO-sum]>10^(-8) && n<3000),
      mu=MoebiusMu[n];
      If[mu=!=0,sumO=sum;
      sum+=mu*If[Re[n*s]<1/2,Log[Zeta[n*s]]+2 Pi*I*countCut[n*s],
        Log[Zeta[n*s]]]/n];
      n++];sum],
  {{countCut[_],_Integer}}];

(* Branc cut counting *)
countCut=Compile[{{s,_Complex}},
  Module[{val,valNext,sR=Re[s],count=0},
    valNext=Arg[Zeta[sR+Im[s]*I]];
    While[sR<=0.5,
      val=valNext;
      sR+=stepDistance[val,sR];
      valNext=Arg[Zeta[sR+Im[s]*I]];
      count+=If[Abs[valNext-val]<6,0,Sign[valNext-val]]];
    count],
  {{stepDistance[_,_],_Real}}];

stepDistance=Compile[{{v,_Real},{sR,_Real}},
  Module[{step1,step0},
    {step1,step0}=If[sR<0.45,{0.2,0.01},{0.05,0.002}];
    step0+(step1-step0)*(1-Abs[v/Pi])^2]];

(********** Prime zeta function : Non horizontal branch cut lines **********)

PrimeNumberZetaP2[s_]:=
  compilePrimeZeta2[s+0.0*I]/;(Re[s]>0 && Precision[s]<Infinity)

compilePrimeZeta2=Compile[{{s,_Complex}},
  Module[{sum=0+0.0*I,sumO=1+0.0*I,n=1,mu},
    While[(Abs[sumO-sum]>10^(-8) && n<3000),
      mu=MoebiusMu[n];
      If[mu=!=0,sumO=sum;sum+=mu*Log[Zeta[n*s]]/n];
      n++];
    sum]];



(*****************************************************************************)
(*                             RamanujanLTau[s]                              *)
(*****************************************************************************)

RamanujanLTau[s_]:=
  Module[{sum=0,sumOld=-1,n=1,nMax=100000},
    While[Abs[sum-sumOld]>10^-12 && n<nMax,
      sumOld=sum;
      sum+=tauList[nMax][[n]]/(n^s);
      n++];
    sum]/;
  (Precision[s]<Infinity && Re[s]>=7)

RamanujanLTau[s_]:=
  If[Abs[Im[s]]<20,
    (termRamanujanL1[s]+termRamanujanL2[s])/Gamma[s],
    RamanujanTauL[s]]/;
  (Precision[s]<Infinity && 6<=Re[s]<7)

RamanujanLTau[s_]:=
  (2 Pi)^(2 s-12)*Gamma[12-s]/Gamma[s]*RamanujanLTau[12-s]/;
  (Precision[s]<Infinity && Re[s]<6)

(* Sub functions *)
termRamanujanL1[s_]:=
  Module[{sum=0,sumOld=-1,n=1,nMax=100000},
    While[Abs[sum-sumOld]>10^-12 && n<nMax,
      sumOld=sum;
      sum+=tauList[nMax][[n]]*Gamma[s,n]/(n^s);
      n++];
    sum];

termRamanujanL2[s_]:=
  Module[{sum=0,sumOld=-1,n=1,nMax=100000},
    While[Abs[sum-sumOld]>10^-12 && n<nMax,
      sumOld=sum;
      sum+=tauList[nMax][[n]]*Gamma[12-s,4*Pi^2*n]/(n^(12-s));
      n++];
    (2 Pi)^(2 s-12)*sum];

(* List of Ramanujan-tau coefficients *)
tauList[nMax_]:=
  tauList[nMax]=RamanujanTau[Range[nMax]];



(*****************************************************************************)
(*                         RamanujanSiegelTheta[t]                           *)
(*****************************************************************************)

RamanujanSiegelTheta[t_]:=RamanujanTauTheta[t];



(*****************************************************************************)
(*                           RamanujanSiegelZ[t]                             *)
(*****************************************************************************)

RamanujanSiegelZ[t_] :=
  Module[{val},
    val=RamanujanLTau[6+I*t]*Exp[I*RamanujanTauTheta[t]];
    If[Abs[Im[t]]>0,val,Re[val]]]/;
  (NumberQ[t] && Precision[t]<Infinity)



(*****************************************************************************)
(*                         EichlerEllipticCurveL[s]                          *)
(*****************************************************************************)

EichlerEllipticCurveL[s_]:=
  Module[{nMax=10000,coefL,clLen},
    coefL=coefEichlerECLList[nMax];
    clLen=Length[coefL];
    coefL.(Range[clLen]^-s)]/;
  (Precision[s]<Infinity && Re[s]>=1 && Not[1<=Re[s]<2 && Abs[Im[s]]<=8])

EichlerEllipticCurveL[s_]:=
  (termEichlerECL1[s]+termEichlerECL2[s])/Gamma[s]/;
  (Precision[s]<Infinity && 1<=Re[s]<2 && Abs[Im[s]]<=8)

EichlerEllipticCurveL[s_]:=
  11^(1-s)*(2 Pi)^(2 s-2)*Gamma[2-s]/Gamma[s]*EichlerEllipticCurveL[2-s]/;
  (Precision[s]<Infinity && Re[s]<1)

(* Sub functions *)
termEichlerECL1[s_]:=
  Module[{nMax=10000,coefL,clLen},
    coefL=Take[coefEichlerECLList[nMax],Ceiling[nMax/100]];
    clLen=Length[coefL];
    (coefL*(Range[clLen]^-s)).Gamma[s,2 Pi*Range[clLen]/Sqrt[11]]];

termEichlerECL2[s_]:=
  Module[{nMax=10000,coefL,clLen},
    coefL=Take[coefEichlerECLList[nMax],Ceiling[nMax/100]];
    clLen=Length[coefL];
    (coefL*(Range[clLen]^(s-2))).Gamma[2-s,2 Pi*Range[clLen]/Sqrt[11]]]*
  11^(1-s)*(2 Pi)^(2 s-2);

(* List of Fourier coefficients of the Cusp-form *)
coefEichlerECLList[nMax_]:=
  coefEichlerECLList[nMax]=
  Module[{q},
    Rest[CoefficientList[Normal[Series[q*(QPochhammer[q]*QPochhammer[q^11])^2,
    {q,0,nMax}]],q]]];

EichlerCoefficientC[n_Integer]:=
  Module[{step=1000,list,len},
    list=coefEichlerECLList[step*Ceiling[n/1000]];
    len=Length[list];
    Join[list,Table[0,{n-len}]]][[n]]/;(n>0)



(*****************************************************************************)
(*                             FibonacciZeta[z]                              *)
(*****************************************************************************)

FibonacciZeta[z_]:=
  Module[{s=0,so=-1,k=1},
  While[so!=s,so=s;s+=Fibonacci[k]^(-z);k++];s]/;
  (NumericQ[z] && Re[z]>=0.5)

FibonacciZeta[z_]:=
  Module[{s=0,so=-1,k=0},
  While[so!=s,so=s;
    s+=SetPrecision[Binomial[-z,k]/(N[GoldenRatio]^(z+2 k)+(-1)^(k+1)),100];
    k++];s]*5^(z/2)/;
  (NumericQ[z] && Re[z]<0.5)



(**************************************************************************)
(*                            RiemannPrimeR[z]                            *)
(**************************************************************************)

RiemannPrimeR[z_]:=RiemannR[z]/;(NumericQ[z])



(**************************************************************************)
(*                           RiemannPrimeRExp[z]                          *)
(**************************************************************************)

RiemannPrimeRExp[z_]:=
  Module[{s=0,so=1,k=2,g1=1,prec,zp,zf},
    prec=Max[200,Ceiling[10+0.4135*Abs[z]+1.0776*Sqrt[Abs[z]]]];
    zp=SetPrecision[z,prec];
    zf=zp;
    While[so!=s,so=s;
      s+=(zf/g1/(k-1)/Zeta[k]);
      g1*=k;zf*=zp;k++];
    s+1]/;
  (Not[Abs[z]>10 && Re[z]<-2] && NumericQ[z])

RiemannPrimeRExp[z_]:=
  1/Pi*trivialZeroTerm[z]+1/2*nonTrivialZeroTerm[z]/;
  (Abs[z]>10 && Re[z]<-2 && NumericQ[z])

trivialZeroTerm[z_]:=
  Module[{s=0,so=1,k=1,zf,mf},mf=-1;zf=(2 Pi/z)^3;
    While[(so!=s && k<takeTermFunc[z]),so=s;
    s+=(mf*zf/(2*k+1)/Zeta[2*k+1]);
    mf*=(-1);zf*=(2 Pi/z)^2;k++];s];

takeTermFunc[z_]:=Min[340,Floor[20+5000000/Abs[z]]];

nonTrivialZeroTerm[z_]:=
  Module[{kMax=340},
    Total[Map[(-2 Pi/z)^#/(#*Cos[# Pi/2])&,zetaZeroList[kMax]]/
      zetaPrimeOnZeta0[kMax]]];

zetaPrimeOnZeta0[k_]:=
  zetaPrimeOnZeta0[k]=
  Map[zetaPrimeVal[#]&,zetaZeroList[k]];



(**************************************************************************)
(*                         RiemannExpIntegral[z]                          *)
(**************************************************************************)

RiemannExpIntegral[z_]:=
  Module[{s=0,so=1,k=1,zp,zf,z2},
    zp=SetPrecision[z,50];
    zf=-zp/(Pi^2);
    z2=-(zp/Pi)^2;
    While[so!=s,so=s;
      s+=(Zeta[2*k]*zf/(2*k-1));
      zf*=z2;k++];
    s+(Log[z]+1/z)/2-constantREI]/;
  (Abs[z]<1 && Re[z]>0 && NumericQ[z])

RiemannExpIntegral[z_]:=
  Module[{s=0,so=1,k=1,prec,zp},
    prec=Max[50,Ceiling[Abs[z]]];
    zp=SetPrecision[z,prec];
    While[so!=s,so=s;
      s+=ExpIntegralE[1,2*k*zp];
      k++];
    N[s]]/;
  (Abs[z]>=1 && Re[z]>0 && NumericQ[z])

RiemannExpIntegral[z_]:=
  Module[{s=0,so=1,k=1,prec,zp},
    prec=Max[50,Ceiling[Abs[z]]];
    zp=SetPrecision[z,prec];
    While[Chop[so-s,10^-5]=!=0,so=s;
      s+=(-originalArcTan[zp/(k*Pi)]/(k*Pi));
      k++];
    s+(Log[z]+1/z)/2-constantREI]/;
  (Re[z]<=0 && NumericQ[z])

constantREI=0.2837571104739336567684576306353281403;

originalArcTan[z_]:=
  If[Re[z]>0 || (Re[z]<=0 && -1<Im[z]<1),
    ArcTan[z],ArcTan[z]+Pi];



(**************************************************************************)
(*                         RiemannLogIntegral[z]                          *)
(**************************************************************************)

RiemannLogIntegral[z_]:=
  RiemannExpIntegral[Log[z]]/;(NumericQ[z])



(**************************************************************************)
(*                     RiemannVonMangoldtPi[k,kMax,x]                     *)
(**************************************************************************)

RiemannVonMangoldtPi[k_Integer,kMax_Integer,x_]:=
  RiemannR[x]-
  Total[Map[zetaZeroTermRM[#,x]&,zetaZeroListUpper[kMax][[1;;k]]]]+
  correctionTermRM[x]/;
  (0<=k<=kMax && NumericQ[x])

correctionTermRM[x_]:=
  1/2-1/Log[x]-ArcTan[Log[x]/Pi]/Pi/;
  (NumericQ[x])

zetaZeroTermRM[rho_,x_]:=
  Module[{n,selN,selMu},
    n=1+Ceiling[Log[2,x]];
    selN=Select[Range[n],SquareFreeQ];
    selMu=Map[N[MoebiusMu[#]/#]&,selN];
    selMu.Map[2*Re[ExpIntegralEi[rho/#*Log[x]]]&,selN]]/;
  (NumericQ[rho] && NumericQ[x])



(**************************************************************************)
(*                  RiemannVonMangoldtPiExact[k,kMax,x]                   *)
(**************************************************************************)

RiemannVonMangoldtPiExact[k_Integer,kMax_Integer,x_]:=
  RiemannR[x]-
  2*Total[Map[Re[RiemannPrimeRExp[#*Log[x]]]&,zetaZeroListUpper[kMax][[1;;k]]]]+
  correctionTermRM[x]/;
  (0<=k<=kMax && NumericQ[x])



(**************************************************************************)
(*            ChebyshevPrimeTheory[k,kMax,x],  ChebyshevPsi[x]            *)
(**************************************************************************)

(*** Chebyshev's approximation of Psi(x) ***)

ChebyshevPrimeTheory[k_Integer,kMax_Integer,x_]:=
  x-Log[2*Pi]-Log[1-1/(x^2)]/2-Sqrt[x]*
    Total[Map[(Cos[#*Log[x]]+2*#*Sin[#*Log[x]])/(1/4+#^2)&,imZeta0LUp[kMax][[1;;k]]]]/;
  (0<=k<=kMax && NumericQ[x])

imZeta0LUp[kMax_Integer]:=
  imZeta0LUp[kMax]=Im[zetaZeroListUpper[kMax]];

(*** Chebyshev's function Psi(x) ***)

ChebyshevPsi[x_]:=
  If[x<2,0,Total[MangoldtLambda[Range[Floor[x]]]]]/;
  (NumericQ[x])



(*****************************************************************************)
(*                              EulerSumH[s,z]                               *)
(*****************************************************************************)

(************ Normal case of Euler sums : H(s)= H(s,1) ************)

EulerSumH[s_]:=
  Module[{kMax=8,nMax=8},
    EulerGamma*Zeta[s]+Zeta[s+1]/2-zetaPrimeVal[s]+
    Total[Map[Zeta[1-2 #]*Zeta[s+2 #]&,N[Range[kMax]]]]+
    Total[Map[integValueESH1[#,kMax]/(#^s)&,Range[nMax]]]]/;
  (Precision[s]<Infinity)

integValueESH1[n_Integer,k_Integer]:=
  integValueESH1[n,k]=compileIntValESH1[n,k];

compileIntValESH1=
  Compile[{{n,_Integer},{k,_Integer}},
    Sum[
      Sum[Binomial[2 k+1,h]*BernoulliB[h]*hyp2F1ValueESH1[k,h,m],{h,0,2 k+1}],
      {m,n,n+10}]];

hyp2F1ValueESH1=
  Compile[{{k,_Integer},{h,_Integer},{m,_Integer}},
    Module[{nuM=N[m]},
     (-1/nuM)^(-2 k+h)*nuM^(-2 k)*Beta[-1/nuM,2 k-h+2,-2 k-1]]];

(************ General case of Euler sums : H(s,z) ************)

EulerSumH[s_,z_]:=
  EulerSumH[s]/;
  (Chop[z-1]===0 && Precision[s]<Infinity)

EulerSumH[s_,z_]:=
  Zeta[s-1]/;
  (Chop[z]===0 && Precision[s]<Infinity)

EulerSumH[s_,z_]:=
  Module[{intZ=Floor[z+1/2]},
    Zeta[s+intZ]/2+Zeta[s+intZ-1]/(1-intZ)-
    Total[Map[Binomial[-intZ,2 #-1]*Zeta[1-2 #]*Zeta[s+intZ+2 #-1]&,
      Range[Floor[-intZ/2]]]]]/;
  (Re[z]<0 && Chop[Im[z]]===0 && Chop[z-Floor[z+1/2]]===0 && 
   Precision[s]<Infinity)

EulerSumH[s_,z_]:=
  Module[{kMax=8,nMax=8},
    Zeta[s]*Zeta[z]-Zeta[s+z-1]/(z-1)+Zeta[s+z]/2+
    Total[Map[Binomial[z+2 #-2,2 #-1]*Zeta[1-2 #]*Zeta[s+z+2 #-1]&,N[Range[kMax]]]]+
    Binomial[z+2 kMax,2 kMax+1]*
    Total[Map[integValueESH[#,kMax,z]/(#^s)&,Range[nMax]]]]/;
  (Not[Re[z]<2 && Chop[Im[z]]===0 && Chop[z-Floor[z+1/2]]===0] && 
   Precision[s]<Infinity)

integValueESH[n_Integer,k_Integer,z_]:=
  Module[{val=compileIntValESH[n,k,z+0.*I]},
    If[Abs[Im[z]]>0,val,Re[val]]];

compileIntValESH=
  Compile[{{n,_Integer},{k,_Integer},{z,_Complex}},
    Sum[
      Sum[Binomial[2 k+1,h]*BernoulliB[h]*hyp2F1ValueESH[k,h,m,z],{h,0,2 k+1}],
      {m,n,n+10}]];

hyp2F1ValueESH=
  Compile[{{k,_Integer},{h,_Integer},{m,_Integer},{z,_Complex}},
    Module[{nuM=N[m]},
     (-1/nuM)^(-2 k+h)*nuM^(1-z-2 k)*Beta[-1/nuM,2 k-h+2,-2 k-z]]];



(*****************************************************************************)
(*                                 RieszR[z]                                 *)
(*****************************************************************************)

RieszR[z_]:=
  Sqrt[Pi]*rieszR1[z]+rieszR2[z]/2/;
  (Re[z]>5 && Precision[z]<Infinity)

RieszR[z_]:=
  6*z/(Pi^2)+
  z*Total[Map[MoebiusMu[#]/N[#^2]*(Exp[-z/N[#^2]]-1)&,Range[500]]]/;
  (Re[z]<=5 && Precision[z]<Infinity)

(********** Sub codes **********)

rieszR1[z_]:=
  Module[{k=200,powerL},
    powerL=Map[(-z)^(-#)&,Range[k]];
    Dot[rieszR1Coeff[k],powerL]];

rieszR1Coeff[k_]:=
  rieszR1Coeff[k]=
  Map[Pi^(2 #)/(Gamma[#+1/2] Zeta[2 #+1])&,N[Range[k]]];

rieszR2[z_]:=
  Module[{k=150,powerL},
    powerL=Map[z^(#/2)&,zetaZeroList[k]];
    Dot[rieszR2Coeff[k],powerL]];

rieszR2Coeff[k_]:=
  rieszR2Coeff[k]=
  Map[Gamma[1-#/2]/zetaPrimeVal[#]&,zetaZeroList[k]];



(*****************************************************************************)
(*                            HardyLittlewoodR[z]                            *)
(*****************************************************************************)

HardyLittlewoodR[z_]:=
  (Sqrt[Pi]*hardylittlewoodR1[z]+hardylittlewoodR2[z]/2)/Sqrt[z]/;
  (Re[z]>5 && Precision[z]<Infinity)

HardyLittlewoodR[z_]:=
  Total[Map[MoebiusMu[#]/N[#]*Exp[-z/N[#^2]]&,Range[740]]]/;
  (Re[z]<=5 && Precision[z]<Infinity)

(********** Sub codes **********)

hardylittlewoodR1[z_]:=
  Module[{k=200,powerL},
    powerL=Map[(-z)^(-#)&,Range[k]];
    Dot[hlR1Coeff[k],powerL]];

hlR1Coeff[k_]:=
  hlR1Coeff[k]=
  Map[Pi^(2 #)/(#!*Zeta[2 #+1])&,N[Range[k]]];

hardylittlewoodR2[z_]:=
  Module[{k=150,powerL},
    powerL=Map[z^(#/2)&,zetaZeroList[k]];
    Dot[hlR2Coeff[k],powerL]];

hlR2Coeff[k_]:=
  hlR2Coeff[k]=
  Map[Gamma[(1-#)/2]/zetaPrimeVal[#]&,zetaZeroList[k]];



(*****************************************************************************)
(*                          Clausen's function CL2                           *)
(*****************************************************************************)

ClausenCL2[th_]:=GeneralClausenS[2,th];

PeriodicClausenCL2[th_]:=PeriodicGeneralClausenS[2,th];



(*****************************************************************************)
(*                      Generalized Clausen's functions                      *)
(*****************************************************************************)

(*** Non-periodic type of the General Clausen's functions ***)

(* Low precision *)
GeneralClausenC[s_,th_]:=
  Module[{val},
    val=(polyLogExpP[s,th]+polyLogExpM[s,th])/2;
    If[Abs[Im[s]]>0 || Abs[Im[th]]>0 || 
      (N[Im[th]]===0. && Not[0<Re[th]<2 Pi]),val,Re[val]]]/;
  (NumericQ[s] && NumericQ[th])

GeneralClausenS[s_,th_]:=
  Module[{val},
    val=(polyLogExpP[s,th]-polyLogExpM[s,th])/(2*I);
    If[Abs[Im[s]]>0 || Abs[Im[th]]>0 || 
      (N[Im[th]]===0. && Not[0<Re[th]<2 Pi]),val,Re[val]]]/;
  (NumericQ[s] && NumericQ[th])

(* High precision *)
GeneralClausenC[s_,th_,prec_]:=
  Module[{val,sp=SetPrecision[s,prec],tp=SetPrecision[th,prec]},
    val=(polyLogExpP[sp,tp]+polyLogExpM[sp,tp])/2;
    If[Abs[Im[s]]>0 || Abs[Im[th]]>0 || 
      (N[Im[th]]===0. && Not[0<Re[th]<2 Pi]),val,Re[val]]]/;
  (NumericQ[s] && NumericQ[th])

GeneralClausenS[s_,th_,prec_]:=
  Module[{val,sp=SetPrecision[s,prec],tp=SetPrecision[th,prec]},
    val=(polyLogExpP[sp,tp]-polyLogExpM[sp,tp])/(2*I);
    If[Abs[Im[s]]>0 || Abs[Im[th]]>0 || 
      (N[Im[th]]===0. && Not[0<Re[th]<2 Pi]),val,Re[val]]]/;
  (NumericQ[s] && NumericQ[th])

(* Sub code *)
polyLogExpP[s_,th_]:=
  If[nearIntegQ[s],
    If[Im[th]>=0,
      PolyLog[s,Exp[I*th]],
      (2 Pi)^s*Exp[Pi*I*s/2]/Gamma[s]*HurwitzZeta[1-s,th/(2 Pi)]-
        Exp[Pi*I*s]*PolyLog[s,Exp[-I*th]]],
    Gamma[1-s]*(2 Pi)^(s-1)*
      (Exp[Pi*I*(1-s)/2]*HurwitzZeta[1-s,th/(2 Pi)]+
       Exp[Pi*I*(s-1)/2]*HurwitzZeta[1-s,1-th/(2 Pi)])];

polyLogExpM[s_,th_]:=
  If[nearIntegQ[s],
    If[Im[th]<=0,
      PolyLog[s,Exp[-I*th]],
      (2 Pi)^s*Exp[-Pi*I*s/2]/Gamma[s]*HurwitzZeta[1-s,th/(2 Pi)]-
        Exp[-Pi*I*s]*PolyLog[s,Exp[I*th]]],
    Gamma[1-s]*(2 Pi)^(s-1)*
      (Exp[Pi*I*(1-s)/2]*HurwitzZeta[1-s,1-th/(2 Pi)]+
       Exp[Pi*I*(s-1)/2]*HurwitzZeta[1-s,th/(2 Pi)])];

nearIntegQ[s_]:=
  (Chop[Im[s]]===0 && Chop[s-Floor[s+1/2]]===0 && Re[s]>1/2);

(*** Periodic type of the General Clausen's functions ***)

(* Low precision *)
PeriodicGeneralClausenC[s_,th_]:=
  Module[{reth=Re[th]-2 Pi*Floor[Re[th]/(2 Pi)]},
    GeneralClausenC[s,reth+I*Im[th]]]/;
  (NumericQ[s] && NumericQ[th])

PeriodicGeneralClausenS[s_,th_]:=
  Module[{reth=Re[th]-2 Pi*Floor[Re[th]/(2 Pi)]},
    GeneralClausenS[s,reth+I*Im[th]]]/;
  (NumericQ[s] && NumericQ[th])

(* High precision *)
PeriodicGeneralClausenC[s_,th_,prec_]:=
  Module[{reth=Re[th]-2 Pi*Floor[Re[th]/(2 Pi)]},
    GeneralClausenC[s,reth+I*Im[th],prec]]/;
  (NumericQ[s] && NumericQ[th])

PeriodicGeneralClausenS[s_,th_,prec_]:=
  Module[{reth=Re[th]-2 Pi*Floor[Re[th]/(2 Pi)]},
    GeneralClausenS[s,reth+I*Im[th],prec]]/;
  (NumericQ[s] && NumericQ[th])



(******************************************************************************)
(*                                 RogersL[z]                                 *)
(******************************************************************************)

RogersL[z_]:=
  PolyLog[2,z]+Log[z]*Log[1-z]/2/;
  (Precision[z]<Infinity)



(******************************************************************************)
(*                           ArcTanIntegralTi[s,z]                            *)
(******************************************************************************)

(* Low precision *)
ArcTanIntegralTi[s_,z_]:=
  (analyticPolyLog[s,I*z]-analyticPolyLog[s,-I*z])/(2*I)/;
  (NumericQ[s] && NumericQ[z])

(* High precision *)
ArcTanIntegralTi[s_,z_,prec_]:=
  Module[{sp=SetPrecision[s,prec],zp=SetPrecision[z,prec]},
    (analyticPolyLog[sp,I*zp]-analyticPolyLog[sp,-I*zp])/(2*I)]/;
  (NumericQ[s] && NumericQ[z])

analyticPolyLog[s_,z_]:=
  If[Re[s]<0 && (Abs[Im[z]]>0 || Re[z]<=0),
    (2 Pi)^s/Gamma[s]*Exp[Pi*I*s/2]*HurwitzZeta[1-s,Log[-z]/(2 Pi*I)+1/2]-
      Exp[Pi*I*s]*PolyLog[s,1/z],
    PolyLog[s,z]];



(******************************************************************************)
(*                             DebyeD1[s,z,type]                              *)
(******************************************************************************)

(*-------------- Common --------------*)

DebyeD1[s_,z_]:=DebyeD1[s,z,1];

DebyeD1[s_,z_,type_]:=
  ComplexInfinity/;
  (MemberQ[{0.,0.+0.*I},N[s]] && NumericQ[s] && NumericQ[z])

realValF[s_,z_,val_]:=
  If[N[Abs[Im[s]]]===0. && N[Abs[Im[z]]]===0. && (Re[z]>0 || non0IntegQ[s]),
  Re[val],val];

non0IntegQ[s_]:=
  (Chop[s-Floor[s],10^(-17)]===0 && N[Im[s]]===0. && Abs[s]>0);

(*-------------- Type-1 --------------*)

DebyeD1[s_,z_,1]:=
  If[Re[z]>0,initDebyeD1[s,z],
    If[-2*Pi<=Im[z]<2*Pi && Re[z]<0,
      DebyeD1[s,z,2],ndsDebyeD1[s,z,1]]]/;
  (Not[MemberQ[{0.,0.+0.*I},N[s]]] && NumericQ[s] && NumericQ[z])

ndsDebyeD1[s_,z_,1]:=
  Module[{theta,sol,f,x,shift=1+(1+2*Floor[Im[z]/(2*Pi)])*Pi*I},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-(Exp[I*theta]*x+shift)^s/
      (Gamma[s+1]*(Exp[Exp[I*theta]*x+shift]-1))==0,
      f[0]==initDebyeD1[s,shift]},f,{x,Abs[z-shift]},
      MaxStepSize->0.005];
    realValF[s,z,f[Abs[z-shift]]/.sol[[1]]]];

(*-------------- Type-2 --------------*)

DebyeD1[s_,z_,2]:=
  If[Re[z]>=0,DebyeD1[s,z,1],
    If[Im[z]>=0,
      Exp[Pi*I*s]*DebyeD1[s,-z,1]-z^(s+1)/Gamma[s+2],
      Exp[-Pi*I*s]*DebyeD1[s,-z,1]-z^(s+1)/Gamma[s+2]]]/;
  (Not[MemberQ[{0.,0.+0.*I},N[s]]] && NumericQ[s] && NumericQ[z])

(*-------------- Type-3 --------------*)

DebyeD1[s_,z_,3]:=
  If[Re[z]<0,DebyeD1[s,z,2],
    If[-2*Pi<=Im[z]<2*Pi,
      DebyeD1[s,z,1],ndsDebyeD1[s,z,3]]]/;
  (Not[MemberQ[{0.,0.+0.*I},N[s]]] && NumericQ[s] && NumericQ[z])

ndsDebyeD1[s_,z_,3]:=
  Module[{theta,sol,f,x,shift=-1+(1+2*Floor[Im[z]/(2*Pi)])*Pi*I},
    theta=If[z==shift,0,Arg[z-shift]];
    sol=NDSolve[{Exp[-I*theta]*f'[x]-(Exp[I*theta]*x+shift)^s/
      (Gamma[s+1]*(Exp[Exp[I*theta]*x+shift]-1))==0,
      f[0]==DebyeD1[s,shift,2]},f,{x,Abs[z-shift]},
      MaxStepSize->0.005];
    realValF[s,z,f[Abs[z-shift]]/.sol[[1]]]];

(*---------- Initial values ----------*)

initDebyeD1[s_,z_]:=
  realValF[s,z,compileInitD1[s+0.0*I,z+0.0*I]]/;
  (Re[z]>0 && NumericQ[s] && NumericQ[z])

compileInitD1=Compile[{{s,_Complex},{z,_Complex}},
  Module[{sum=0+0.0*I,sumO=1+0.0*I,n=1,nMax},
    nMax=If[Re[s]>0,500,1000];
    While[(sumO!=sum && n<nMax),sumO=sum;
      sum+=(GammaRegularized[s+1,n*z]*n^(-s-1));
      n++];
    Zeta[s+1]-sum]];



(******************************************************************************)
(*                             DebyeD2[s,z,type]                              *)
(******************************************************************************)

DebyeD2[s_,z_]:=DebyeD2[s,z,1];

DebyeD2[s_,z_,type_]:=
  Which[
    type===1,-Log[1-Exp[-z]],
    type===2,
      If[Re[z]>=0,-Log[1-Exp[-z]],
        If[Im[z]>=0,-Log[1-Exp[z]]+z-Pi*I,-Log[1-Exp[z]]+z+Pi*I]],
    type===3,
      If[Im[z]>0,z-Pi*I-Log[1-Exp[z]],
        If[Im[z]<0,z+Pi*I-Log[1-Exp[z]],-Log[1-Exp[-z]]]]]/;
  (MemberQ[{0.,0.+0.*I},N[s]] && NumericQ[s] && NumericQ[z])

DebyeD2[s_,z_,type_]:=
  Zeta[s+1]-DebyeD1[s,z,type]/;
  (Not[MemberQ[{0.,0.+0.*I},N[s]]] && NumericQ[s] && NumericQ[z])



(******************************************************************************)
(*               GrueneisenG1[s,z,type], GrueneisenG2[s,z,type]               *)
(******************************************************************************)

GrueneisenG1[s_,z_]:=GrueneisenG1[s,z,1];

GrueneisenG1[s_,z_,type_]:=
  DebyeD1[s-1,z,type]-z^s/((Exp[z]-1)*Gamma[s+1])/;
  (NumericQ[s] && NumericQ[z])

GrueneisenG2[s_,z_]:=GrueneisenG2[s,z,1];

GrueneisenG2[s_,z_,type_]:=
  DebyeD2[s-1,z,type]+z^s/((Exp[z]-1)*Gamma[s+1])/;
  (NumericQ[s] && NumericQ[z])



(******************************************************************************)
(*             StroemgrenPhi1[s,z,type], StroemgrenPhi2[s,z,type]             *)
(******************************************************************************)

StroemgrenPhi1[s_,z_]:=StroemgrenPhi1[s,z,1];

StroemgrenPhi1[s_,z_,type_]:=
  (DebyeD1[s-1,z,type]+DebyeD1[s-2,z,type])/2-
  ((1+s/z)*z^s/(Exp[z]-1)+z^s*Exp[z]/((Exp[z]-1)^2))/(2*Gamma[s+1])/;
  (NumericQ[s] && NumericQ[z])

StroemgrenPhi2[s_,z_]:=StroemgrenPhi2[s,z,1];

StroemgrenPhi2[s_,z_,type_]:=
  (DebyeD2[s-1,z,type]+DebyeD2[s-2,z,type])/2+
  ((1+s/z)*z^s/(Exp[z]-1)+z^s*Exp[z]/((Exp[z]-1)^2))/(2*Gamma[s+1])/;
  (NumericQ[s] && NumericQ[z])



(*****************************************************************************)

End[]

Protect[RiemannXi, PeriodicZetaF, ImprovedHurwitzZeta, 
        ImprovedDirichletL, GaussSum, ConjugateChiIndex, 
        RealDirichletCharacterQ, PrimitiveDirichletCharacterQ, 
        GrundzahlD, GrundzahlDQ, QuadraticFieldDedekindZeta, 
        CyclotomicFieldDedekindZeta, ImprovedLerchPhi, 
        ZetaPrime, StieltjesFGamma, SitaramachandraraoDelta, 
        ZetaZeta, LehmerZetaZeta, 
        BellExponentB, BellExponentB2, 
        PrimeNumberZetaP, PrimeNumberZetaP2, 
        RamanujanLTau, RamanujanSiegelTheta, RamanujanSiegelZ, 
        EichlerEllipticCurveL, EichlerCoefficientC, FibonacciZeta, 
        RiemannPrimeR, RiemannPrimeRExp, RiemannExpIntegral, 
        RiemannLogIntegral, RiemannVonMangoldtPi, 
        RiemannVonMangoldtPiExact, ChebyshevPrimeTheory, ChebyshevPsi, 
        EulerSumH, RieszR, HardyLittlewoodR, 
        ClausenCL2, PeriodicClausenCL2, 
        GeneralClausenC, GeneralClausenS, 
        PeriodicGeneralClausenC, PeriodicGeneralClausenS, 
        RogersL, ArcTanIntegralTi, DebyeD1, DebyeD2, 
        GrueneisenG1, GrueneisenG2, StroemgrenPhi1, StroemgrenPhi2]

EndPackage[]
